@@ -34,7 +34,7 @@ import json
 
 from base64 import b64encode
 from netrc import netrc
-from pathlib import Path
+from pathlib import Path, PurePath
 
 from . import WrapMode
 from .. import coredata
@@ -54,8 +54,7 @@ try:
 except ImportError:
     has_ssl = False
 
-REQ_TIMEOUT = 600.0
-SSL_WARNING_PRINTED = False
+REQ_TIMEOUT = 30.0
 WHITELIST_SUBDOMAIN = 'wrapdb.mesonbuild.com'
 
 ALL_TYPES = ['file', 'git', 'hg', 'svn']
@@ -96,10 +95,7 @@ def open_wrapdburl(urlstring: str, allow_insecure: bool = False, have_opt: bool
         raise WrapException(f'SSL module not available in {sys.executable}: Cannot contact the WrapDB.{insecure_msg}')
     else:
         # following code is only for those without Python SSL
-        global SSL_WARNING_PRINTED  # pylint: disable=global-statement
-        if not SSL_WARNING_PRINTED:
-            mlog.warning(f'SSL module not available in {sys.executable}: WrapDB traffic not authenticated.')
-            SSL_WARNING_PRINTED = True
+        mlog.warning(f'SSL module not available in {sys.executable}: WrapDB traffic not authenticated.', once=True)
 
     # If we got this far, allow_insecure was manually passed
     nossl_url = url._replace(scheme='http')
@@ -228,6 +224,8 @@ class PackageDefinition:
                 self.diff_files.append(path)
 
     def parse_provide_section(self, config: configparser.ConfigParser) -> None:
+        if config.has_section('provides'):
+            raise WrapException('Unexpected "[provides]" section, did you mean "[provide]"?')
         if config.has_section('provide'):
             for k, v in config['provide'].items():
                 if k == 'dependency_names':
@@ -287,6 +285,7 @@ class Resolver:
     wrap_mode: WrapMode = WrapMode.default
     wrap_frontend: bool = False
     allow_insecure: bool = False
+    silent: bool = False
 
     def __post_init__(self) -> None:
         self.subdir_root = os.path.join(self.source_dir, self.subdir)
@@ -434,10 +433,10 @@ class Resolver:
                 # Write a dummy wrap file in main project that redirect to the
                 # wrap we picked.
                 with open(main_fname, 'w', encoding='utf-8') as f:
-                    f.write(textwrap.dedent('''\
+                    f.write(textwrap.dedent(f'''\
                         [wrap-redirect]
-                        filename = {}
-                        '''.format(os.path.relpath(self.wrap.filename, self.subdir_root))))
+                        filename = {PurePath(os.path.relpath(self.wrap.filename, self.subdir_root)).as_posix()}
+                        '''))
         else:
             # No wrap file, it's a dummy package definition for an existing
             # directory. Use the source code in place.
@@ -697,7 +696,8 @@ class Resolver:
                 return hashvalue, tmpfile.name
             sys.stdout.flush()
             progress_bar = ProgressBar(bar_type='download', total=dlsize,
-                                       desc='Downloading')
+                                       desc='Downloading',
+                                       disable=(self.silent or None))
             while True:
                 block = resp.read(blocksize)
                 if block == b'':
@@ -797,14 +797,17 @@ class Resolver:
             if not path.exists():
                 raise WrapException(f'Diff file "{path}" does not exist')
             relpath = os.path.relpath(str(path), self.dirname)
-            if PATCH:
-                cmd = [PATCH, '-f', '-p1', '-i', relpath]
-            elif GIT:
-                # If the `patch` command is not available, fall back to `git
-                # apply`. The `--work-tree` is necessary in case we're inside a
+            if GIT:
+                # Git is more likely to be available on Windows and more likely
+                # to apply correctly assuming patches are often generated by git.
+                # See https://github.com/mesonbuild/meson/issues/12092.
+                # The `--work-tree` is necessary in case we're inside a
                 # Git repository: by default, Git will try to apply the patch to
                 # the repository root.
                 cmd = [GIT, '--work-tree', '.', 'apply', '-p1', relpath]
+            elif PATCH:
+                # Always pass a POSIX path to patch, because on Windows it's MSYS
+                cmd = [PATCH, '-f', '-p1', '-i', str(Path(relpath).as_posix())]
             else:
                 raise WrapException('Missing "patch" or "git" commands to apply diff files')
 
